﻿# -*- coding: utf-8 -*-
"""
An error occurs if control characters are included.
"""

import cse
import cseMessageBox
import re

# ASCII control characters excluding \t (\x09), \n (\x0a), \r (\x0d)
rePattern = re.compile(r"[\x00-\x08\x0b\x0c\x0e-\x1f\x7f]")


# Determine whether control characters are included.
def hasCtrlChars(cont):
  if str(cont.type) == "text" or str(cont.type) == "multitext":
    if rePattern.search(cont.Value) is not None:
      return True
    else:
      return False
  elif str(cont.type) == "list":
    strt = ''.join([listItem for listItem in cont.Value])
    if rePattern.search(strt) is not None:
      return True
    else:
      return False


# Remove control characters.
def removeCtrlChars(cont):
  if str(cont.type) == "text" or str(cont.type) == "multitext":
    cont.Value = rePattern.sub('', cont.Value)
  elif str(cont.type) == "list":
    for i in range(len(cont.Value)):
      cont.Value[i] = rePattern.sub('', cont.Value[i])


# Output Sheet Check Member Function
def CheckOutSheet(s, e):
  # Plug-In Execution Event Data
  v = e.Value
  
  # Error Type
  errtype = "PlgErr_ChkCtrlChar_ContainsCtrlChar"
  
  # Loop with parent and child implementation elements
  for sti in v.sheets:
    if sti.IsShown: # Only visible checksheets
      for contset in cse.getcontssets(sti.Sheet, False):
        for flattenCont in contset:
          if hasCtrlChars(flattenCont):
            # An error occurs for items including control characters.
            v.errors.append({'type': errtype, 'path': cse.deparsepath(flattenCont)})


# Intput Sheet Check Member Function
def CheckImportedSheet(s, e):
  # Plug-In Execution Event Data
  v = e.Value
  
  # Error String Base
  errMsgKey = "PlgMsg_ChkCtrlChar_RemoveCtrlChar"
  errMsgBase = cse.getapplang(errMsgKey)
  if not errMsgBase:
    # Error if null character
    raise ('Lang key "%s" is not found.' % errMsgKey)

  # List of elements with control characters mixed together
  havingCtrlCharCells = [flattenCont for sti in v.sheets for contset in cse.getcontssets(sti.Sheet, False) for flattenCont in contset if hasCtrlChars(flattenCont)]
  
  # If a list exists and Yes is selected in the dialog box, all control characters are deleted.
  if len(havingCtrlCharCells) > 0:
    # Create error message with first 32 elements of cell containing control characters.
    errtargets = "\n".join("  " + cse.getcellpathtitle(str(cont.Path)) for cont in havingCtrlCharCells[:32])
    errmsg = errMsgBase % errtargets
    
    if len(havingCtrlCharCells) > 32:
      # Display omitted numbers
      errmsg += "\n  and more... (%d)" % (len(havingCtrlCharCells) - 32)
    
    if cseMessageBox.askyesno("", errmsg):
      for cont in havingCtrlCharCells:
        removeCtrlChars(cont)


cse.addhandler("RequestCheckOutSheet", CheckOutSheet)
cse.addhandler("AfterFileImported", CheckImportedSheet)
